/*
The functions coded by Alexey Pechnikov (pechnikov@mobigroup.ru) and tested on linux only.
The code is public domain. 

Compile as
     gcc -fPIC -lm -shared auxfunc.c -o libsqliteauxfunc.so
*/

#if !defined(SQLITE_CORE) || defined(SQLITE_ENABLE_AUXFUNC)

#include <string.h>
#include "auxfunc.h"

#ifndef SQLITE_CORE
  #include "sqlite3ext.h"
  SQLITE_EXTENSION_INIT1
#else
  #include "sqlite3.h"
#endif

/*
** Implementation of the counter(X) function.  If X is an integer
** constant, then the first invocation will return X.  The second X+1.
** and so forth.  Can be used (for example) to provide a sequence number
** in a result set.
*/
static void counterFunc(
  sqlite3_context *pCtx,   /* Function context */
  int nArg,                /* Number of function arguments */
  sqlite3_value **argv     /* Values for all function arguments */
){
  int *pCounter;

  pCounter = (int*)sqlite3_get_auxdata(pCtx, 0);
  if( pCounter==0 ){
    pCounter = sqlite3_malloc( sizeof(*pCounter) );
    if( pCounter==0 ){
      sqlite3_result_error_nomem(pCtx);
      return;
    }
    *pCounter = sqlite3_value_int(argv[0]);
    sqlite3_set_auxdata(pCtx, 0, pCounter, sqlite3_free);
  }else{
    ++*pCounter;
  }
  sqlite3_result_int(pCtx, *pCounter);
}

#if !defined(SQLITE_CORE) || defined(SQLITE_ENABLE_MD5)
static void distinctonFunc(
  sqlite3_context *pCtx, 
  int argc,
  sqlite3_value **argv
){
  int i;
  int result=0;
  char *pHash;
  typedef char MD5Context[88];
  MD5Context ctx;
  char *pDigest;

  if( argc<1 ) return;

  MD5Init(&ctx);
  for(i=1; i<argc; i++){
    if( sqlite3_value_blob(argv[i]) ){
      MD5Update(&ctx, (char*)sqlite3_value_blob(argv[i]), sqlite3_value_bytes(argv[i]));
    }
  }
  pDigest = sqlite3_malloc( 16 );
  if( pDigest==0 ){
    sqlite3_result_error_nomem(pCtx);
    return;
  }
  MD5Final((char*)pDigest,&ctx);

  pHash = (char*)sqlite3_get_auxdata(pCtx, 0);
  if( pHash==0 ){
    sqlite3_set_auxdata(pCtx, 0, pDigest, sqlite3_free);
    result=1;
  }else{
    if (strncmp(pDigest,pHash,16) == 0) { // hashes are equal, it's duplicate
      sqlite3_free(pDigest);
      result=0;
    } else {
      sqlite3_set_auxdata(pCtx, 0, pDigest, sqlite3_free);
      result=1;
    }
  }
  if (result) { // rows are equal, return first argument
    switch ( sqlite3_value_type(argv[0]) ) {
      case SQLITE_INTEGER:
        sqlite3_result_int64(pCtx, sqlite3_value_int64(argv[0]));
        break;
      case SQLITE_TEXT:
        sqlite3_result_text(pCtx, (char*)sqlite3_value_text(argv[0]), sqlite3_value_bytes(argv[0]), SQLITE_STATIC);
        break;
      default:
        sqlite3_result_blob(pCtx, sqlite3_value_blob(argv[0]), sqlite3_value_bytes(argv[0]), 0);
        break;
    }
  }
}
#endif

/* SQLite invokes this routine once when it loads the extension.
** Create new functions, collating sequences, and virtual table
** modules here.  This is usually the only exported symbol in
** the shared library.
*/

int sqlite3AuxfuncInit(sqlite3 *db){
  static const struct {
     char *zName;
     signed char nArg;
     int argType;           /* 1: 0, 2: 1, 3: 2,...  N:  N-1. */
     int eTextRep;          /* 1: UTF-16.  0: UTF-8 */
     void (*xFunc)(sqlite3_context*,int,sqlite3_value **);
  } aFuncs[] = {
	{ "counter",         -1, 0, SQLITE_UTF8,    counterFunc },
#if !defined(SQLITE_CORE) || defined(SQLITE_ENABLE_MD5)
	{ "distincton",      -1, 0, SQLITE_UTF8,    distinctonFunc },
#endif
  };

  int i;
  for(i=0; i<sizeof(aFuncs)/sizeof(aFuncs[0]); i++){
    void *pArg;
    int argType = aFuncs[i].argType;
    pArg = (void*)(int)argType;
    sqlite3_create_function(db, aFuncs[i].zName, aFuncs[i].nArg,
        aFuncs[i].eTextRep, pArg, aFuncs[i].xFunc, 0, 0);
  }

  return 0;
}

#if !SQLITE_CORE
int sqlite3_extension_init(
  sqlite3 *db, 
  char **pzErrMsg,
  const sqlite3_api_routines *pApi
){
  SQLITE_EXTENSION_INIT2(pApi)
  return sqlite3AuxfuncInit(db);
}
#endif

#endif
