/*
The functions was coded by Alexey Pechnikov (pechnikov@mobigroup.ru) and tested on linux only. Tests are writed and performed by Alexander Romanov (romanov@mobigroup.ru).
The code is public domain. 

1) Compile with
   Linux:
     gcc -fPIC -lm -shared billing.c -o libsqlitebilling.so

Use as:

	.load './libsqlitebilling.so'
or
	SELECT load_extension('./libsqlitebilling.so');
or
	SELECT load_extension('/usr/lib/sqlite3/libsqlitebilling.so');

Samples:

	select cisconumbernorm('ton:4,npi:1,pi:0,si:1,','7','495') is null;
	1

[Russian]
	select cisconumbernorm('ton:4,npi:1,pi:0,si:1,#:81074957862434','7','495');
	74957862434

	select cisconumbernorm('ton:4,npi:1,pi:0,si:1,#:8104957862434','7','495');
	74957862434

	select cisconumbernorm('ton:4,npi:1,pi:0,si:1,#:84957862434','7','495');
	74957862434

	select cisconumbernorm('ton:4,npi:1,pi:0,si:1,#:4957862434','7','495');
	74957862434

	select cisconumbernorm('ton:4,npi:1,pi:0,si:1,#:7862434','7','495');
	74957862434

[USA]
	select cisconumbernorm('ton:0,npi:0,#:81019174534420','7','495');
	19174534420

[Ukraine]
	select cisconumbernorm('ton:2,npi:1,#:810380962836581','7','495');
	380962836581

[Europe]
	select cisconumbernorm('ton:0,npi:0,#:81038269253524','7','495');
	38269253524


Function number2direction:

	CREATE TABLE direction_telephony(group_name,name,class,prefix,price,currency);
	INSERT INTO "direction_telephony" VALUES('Группа 1','Москва','МГ','7495','0.5','RUB');
	INSERT INTO "direction_telephony" VALUES('Группа 1','Россия','МГ','7','2.0','RUB');

Direction not found (return NULL):
	select number2direction ('direction_telephony','group_name','prefix','Группа 2','74957123456');

	select number2direction ('direction_telephony','group_name','prefix','Группа 2','74957123456') IS NULL;
	1

Direction found (return rowid):
	select number2direction ('direction_telephony','group_name','prefix','Группа 1','74957862434');
	1

	select number2direction ('direction_telephony','group_name','prefix','Группа 1','78312123456');
	2


	select * from direction_telephony where rowid in (select number2direction ('direction_telephony','group_name','prefix','Группа 1','74957862434'));
	Группа 1|Москва|МГ|7495|0.5|RUB

	select * from direction_telephony where rowid = (select number2direction ('direction_telephony','group_name','prefix','Группа 1','74957862434'));
	Группа 1|Москва|МГ|7495|0.5|RUB

*/

#if !defined(SQLITE_CORE) || defined(SQLITE_ENABLE_BILLING)

#include <stdlib.h>
#include <sys/types.h>
#include <string.h>

#ifndef SQLITE_CORE
  #include "sqlite3ext.h"
  SQLITE_EXTENSION_INIT1
#else
  #include "sqlite3.h"
#endif

#define STREQ(a,b,n) (strncmp(a,b,n)==0)

// number2direction (table,field_group,field_number,group_name,number)
// select number2direction ('direction_telephony','group_name','prefix','Группа 1','74957862434');
static void number2directionFunc(sqlite3_context *context, int argc, sqlite3_value **argv){
	sqlite3 *db;
    sqlite3_stmt *pStmt;        /* A statement */
    int rc;                     /* Result code */
    char *zSql;                 /* An SQL statement */
	const unsigned char *zTable;
	const unsigned char *zFieldGroup;
	const unsigned char *zFieldNumber;
	const unsigned char *zGroup;
	const unsigned char *zNumber;
	sqlite_int64 rowid=0;
	unsigned char *number;
	int len_number;

	if( SQLITE_NULL==sqlite3_value_type(argv[0]) || SQLITE_NULL==sqlite3_value_type(argv[1]) || SQLITE_NULL==sqlite3_value_type(argv[2]) || SQLITE_NULL==sqlite3_value_type(argv[3]) || SQLITE_NULL==sqlite3_value_type(argv[4]) ){
		sqlite3_result_null(context);
		return;
	}
	zTable = sqlite3_value_text(argv[0]);
	zFieldGroup = sqlite3_value_text(argv[1]);
	zFieldNumber = sqlite3_value_text(argv[2]);
	zGroup = sqlite3_value_text(argv[3]);
	zNumber = sqlite3_value_text(argv[4]);

	db = (sqlite3*) sqlite3_context_db_handle(context);
	zSql = sqlite3_mprintf("SELECT rowid FROM %Q WHERE %q=? and %q=? limit 1", zTable, zFieldGroup, zFieldNumber);
    rc = sqlite3_prepare_v2(db, zSql, -1, &pStmt, 0);
	sqlite3_free(zSql);
    if( rc != SQLITE_OK ){
		sqlite3_result_error(context, sqlite3_errmsg(db), -1);
		return;
    }
	int i;
	len_number = strlen(zNumber);
	number = sqlite3_malloc(len_number + 1);
	strncpy((char*) number, (char*) zNumber, len_number);
	for ( i=len_number; i>0; i-- ) {
		sqlite3_bind_text(pStmt, 1, zGroup, -1, SQLITE_STATIC);
		if( number==0 ){
			sqlite3_result_error(context, sqlite3_errmsg(db), -1);
			return;
		}
		*(number+i) = '\0';
		sqlite3_bind_text(pStmt, 2, number, -1, SQLITE_STATIC);
		rc = sqlite3_step(pStmt);
		if( rc == SQLITE_ROW ) {
			rowid    = sqlite3_column_int64(pStmt,0);
		}
		rc = sqlite3_reset(pStmt);
		if (rowid != 0 ) {
			break;
		}
	}
	sqlite3_free(number);
	sqlite3_finalize(pStmt);
	if (rowid != 0 ) {
		sqlite3_result_int64( context, rowid );
		return;
	}
	sqlite3_result_null(context);
}

// cisconumbernorm(cisco_string, prefix_country, prefix_city)
static void cisconumbernormFunc(sqlite3_context *context, int argc, sqlite3_value **argv){
	char *number_in;
	const char *number_prefix;
	const char *number_prefix2;
	unsigned int len_prefix, len_prefix2, len_number;
	char  *number;

	if( SQLITE_NULL==sqlite3_value_type(argv[0]) || SQLITE_NULL==sqlite3_value_type(argv[1]) || SQLITE_NULL==sqlite3_value_type(argv[2])){
		sqlite3_result_null(context);
		return;
	}

	number_in = (unsigned char *)sqlite3_value_text(argv[0]);
	// входная строка пустая, это в принципе ошибочная ситуация, но пусть решает вызывающее приложение, так что вернем NULL
	if( number_in==0 ) {
		sqlite3_result_null(context);
		return;
	}
	number_prefix = sqlite3_value_text(argv[1]);
	len_prefix = strlen((char*)number_prefix);
	number_prefix2 = sqlite3_value_text(argv[2]);
	len_prefix2 = strlen((char*)number_prefix2);
	// ищем разделитель вида #:
	while( *number_in != 0 ){
		if ( STREQ((char*)number_in,"#:",2) ) break;
		number_in++;
	}
	if ( strlen(number_in) > 2 ) {
		number_in+=2; // получили номер
	} else {	// строка опять же некорректна, обработку ситуации оставляем вызывающему приложению, так что вернем NULL
		sqlite3_result_null(context);
		return;
	}
	len_number = strlen((char*)number_in);
	if ( 15 < len_number || 0==len_number) {								// максимально допустимая длина номера E.164 равна 15 знаков
		sqlite3_result_null(context);
		return;
	}
	// определяем тип номера - местный, МГ, МН и добавляем при необходимости префикс страны/города для приведения к E.164
	if ( 7 >= len_number ) {										// местный, не более 7 цифр
		number = sqlite3_mprintf("%s%s%s", number_prefix, number_prefix2, number_in);
	} else if ( 10 == len_number ) {										// МГ
		number = sqlite3_mprintf("%s%s", number_prefix, number_in);
	} else if ( 11 == len_number && STREQ((char*)number_in,"8",1) ) {		// МГ с кодом выхода на АТС
		number = sqlite3_mprintf("%s%s", number_prefix, number_in + 1);
	} else if ( 13 == len_number && STREQ((char*)number_in,"810",3) ) {		// МГ с кодом выхода на АТС
		number = sqlite3_mprintf("%s%s", number_prefix, number_in + 3);
	} else if ( 13 < len_number && STREQ((char*)number_in,"810",3) ) {		// МН с кодом выхода на АТС
		number = sqlite3_mprintf("%s", number_in + 3);
	} else {	// такого номера быть не должно! 
		sqlite3_result_null(context);
		return;
	}
	sqlite3_result_text( context, number, strlen(number), sqlite3_free);
}


// numbernorm(number, prefix_country, prefix_city)
static void numbernormFunc(sqlite3_context *context, int argc, sqlite3_value **argv){
	char *number_in;
	const char *number_prefix;
	const char *number_prefix2;
	unsigned int len_prefix, len_prefix2, len_number;
	char  *number;

	if( SQLITE_NULL==sqlite3_value_type(argv[0]) || SQLITE_NULL==sqlite3_value_type(argv[1]) || SQLITE_NULL==sqlite3_value_type(argv[2])){
		sqlite3_result_null(context);
		return;
	}

	number_in = (unsigned char *)sqlite3_value_text(argv[0]);
	// входная строка пустая, это в принципе ошибочная ситуация, но пусть решает вызывающее приложение, так что вернем NULL
	if( number_in==0 ) {
		sqlite3_result_null(context);
		return;
	}
	number_prefix = sqlite3_value_text(argv[1]);
	len_prefix = strlen((char*)number_prefix);
	number_prefix2 = sqlite3_value_text(argv[2]);
	len_prefix2 = strlen((char*)number_prefix2);
	if ( strlen(number_in) > 2 ) {
		// получили номер
	} else {	// строка некорректна, обработку ситуации оставляем вызывающему приложению, так что вернем NULL
		sqlite3_result_null(context);
		return;
	}
	len_number = strlen((char*)number_in);
	if ( 15 < len_number || 0==len_number) {								// максимально допустимая длина номера E.164 равна 15 знаков
		sqlite3_result_null(context);
		return;
	}
	// определяем тип номера - местный, МГ, МН и добавляем при необходимости префикс страны/города для приведения к E.164
	if ( 7 >= len_number ) {										// местный, не более 7 цифр
		number = sqlite3_mprintf("%s%s%s", number_prefix, number_prefix2, number_in);
	} else if ( 10 == len_number ) {										// МГ
		number = sqlite3_mprintf("%s%s", number_prefix, number_in);
	} else if ( 11 == len_number && STREQ((char*)number_in,"8",1) ) {		// МГ с кодом выхода на АТС
		number = sqlite3_mprintf("%s%s", number_prefix, number_in + 1);
	} else if ( 13 == len_number && STREQ((char*)number_in,"810",3) ) {		// МГ с кодом выхода на АТС
		number = sqlite3_mprintf("%s%s", number_prefix, number_in + 3);
	} else if ( 13 < len_number && STREQ((char*)number_in,"810",3) ) {		// МН с кодом выхода на АТС
		number = sqlite3_mprintf("%s", number_in + 3);
	} else {	// такого номера быть не должно! 
		sqlite3_result_null(context);
		return;
	}
	sqlite3_result_text( context, number, strlen(number), sqlite3_free);
}


/* SQLite invokes this routine once when it loads the extension.
** Create new functions, collating sequences, and virtual table
** modules here.  This is usually the only exported symbol in
** the shared library.
*/

int sqlite3BillingInit(sqlite3 *db){
  static const struct {
     char *zName;
     signed char nArg;
     int argType;           /* 1: 0, 2: 1, 3: 2,...  N:  N-1. */
     int eTextRep;          /* 1: UTF-16.  0: UTF-8 */
     void (*xFunc)(sqlite3_context*,int,sqlite3_value **);
  } aFuncs[] = {
	{ "cisconumbernorm",             3, 0, SQLITE_UTF8,    cisconumbernormFunc },
	{ "numbernorm",                  3, 0, SQLITE_UTF8,    numbernormFunc },
	{ "number2direction",            5, 0, SQLITE_UTF8,    number2directionFunc },
  };

  int i;
  for(i=0; i<sizeof(aFuncs)/sizeof(aFuncs[0]); i++){
    void *pArg;
    int argType = aFuncs[i].argType;
    pArg = (void*)(int)argType;
    sqlite3_create_function(db, aFuncs[i].zName, aFuncs[i].nArg,
        aFuncs[i].eTextRep, pArg, aFuncs[i].xFunc, 0, 0);
  }

  return 0;
}

#if !SQLITE_CORE
int sqlite3_extension_init(
  sqlite3 *db, 
  char **pzErrMsg,
  const sqlite3_api_routines *pApi
){
  SQLITE_EXTENSION_INIT2(pApi)
  return sqlite3BillingInit(db);
}
#endif

#endif
