/*
 *      Pragma wrapper extension for SQLite
 *
 *      Copyright 2009 Alexey Pechnikov <pechnikov@mobigroup.ru>
 *      The code is public domain.
 */

/* 
Compile as
     gcc -shared pragmacols.c -o libsqlitepragmacols.so
*/

#if !defined(SQLITE_CORE) || defined(SQLITE_ENABLE_PRAGMAWRAPPER)

#include <stdlib.h>

#ifndef NDEBUG
    #include <string.h>
#endif 

#ifndef SQLITE_CORE
    #include <sqlite3ext.h>
    SQLITE_EXTENSION_INIT1
#else
    #include "sqlite3.h"
#endif

static void pragma_cols(sqlite3_context *context, int argc,
    sqlite3_value **argv)
{
    int rc, i=0;
    sqlite3_stmt *stmt;
    sqlite3 *db;
    char *str=NULL, *pragma;

    db = sqlite3_context_db_handle(context);

    if (argc != 1 || (sqlite3_value_type(argv[0]) != SQLITE_TEXT)) {
        sqlite3_result_error(context, "Function may have single argument with PRAGMA statement", -1);
        return;
    }

    rc = sqlite3_prepare_v2(db, (char*)sqlite3_value_text(argv[0]), -1, &stmt, NULL);
    if (rc != SQLITE_OK) {
        sqlite3_result_error_code(context, rc);
        return;
    }

    str = sqlite3_mprintf("%s", sqlite3_column_name(stmt,0));
    for (i=1;i<sqlite3_column_count(stmt);i++) {
        str = sqlite3_mprintf("%s, %s", str, sqlite3_column_name(stmt,i));
    }
    sqlite3_finalize(stmt);

    if (strlen(str)) {
        sqlite3_result_text( context, str, strlen(str), sqlite3_free);
    } else {
        sqlite3_result_null( context );
        sqlite3_free(str);
    }
}

#if !SQLITE_CORE
static
#endif
int sqlite3PragmawrapperInit(sqlite3 *db)
{
    int rc;
    rc = sqlite3_create_function(db, "pragma_cols"  , 1, SQLITE_UTF8, 0, pragma_cols  , 0, 0);
    return rc;
}

#if !SQLITE_CORE
int sqlite3_extension_init(sqlite3 *db, char **pzErrMsg,
                        const sqlite3_api_routines *pApi)
{
    SQLITE_EXTENSION_INIT2(pApi)
    return sqlite3PragmawrapperInit(db);
}
#endif

#endif
