/*
 *      Pragma wrapper extension for SQLite
 *
 *      Copyright 2009 Alexey Pechnikov <pechnikov@mobigroup.ru>
 *      The code is public domain.
 */

/* 
Compile as
     gcc -shared pragmawrapper.c -o libsqlitepragmawrapper.so
*/

#if !defined(SQLITE_CORE) || defined(SQLITE_ENABLE_PRAGMAWRAPPER)

#include <stdlib.h>

#ifndef NDEBUG
    #include <string.h>
#endif 

#ifndef SQLITE_CORE
    #include <sqlite3ext.h>
    SQLITE_EXTENSION_INIT1
#else
    #include "sqlite3.h"
#endif

static void database_list(sqlite3_context *context, int argc,
    sqlite3_value **argv)
{
    int rc;
    sqlite3_stmt *stmt, *pStmt;
    sqlite3 *db;
    char *zSql;                 /* An SQL statement */

    db = sqlite3_context_db_handle(context);

    rc = sqlite3_exec(db, "CREATE TEMP TABLE IF NOT EXISTS database_list(seq integer, name text, file text); \
                           DELETE FROM temp.database_list;",
                          NULL, NULL, NULL);

    /*
    PRAGMA database_list returns:

    [0] = seq
    [1] = name
    [2] = file
    */
    rc = sqlite3_prepare_v2(db, "PRAGMA database_list", -1, &stmt, NULL);

    if (rc != SQLITE_OK) {
        sqlite3_result_error_code(context, rc);
        return;
    }

    rc = sqlite3_prepare(db, "INSERT INTO temp.database_list VALUES (?, ?, ?)", -1, &pStmt, 0);
    if( rc != SQLITE_OK ){
        sqlite3_result_error_code(context, rc);
        return;
    }

    while ((rc = sqlite3_step(stmt)) != SQLITE_DONE) {
        if (rc != SQLITE_ROW) break;
        sqlite3_bind_int64(pStmt, 1, sqlite3_column_int64(stmt, 0));
        sqlite3_bind_text (pStmt, 2, sqlite3_column_text(stmt, 1), -1, SQLITE_TRANSIENT);
        sqlite3_bind_text (pStmt, 3, sqlite3_column_text(stmt, 2), -1, SQLITE_TRANSIENT);
        sqlite3_step(pStmt);
        rc = sqlite3_reset(pStmt);
    }
    sqlite3_finalize(pStmt);
    sqlite3_finalize(stmt);

    sqlite3_result_null(context);
}

#if !SQLITE_CORE
static
#endif
int sqlite3PragmawrapperInit(sqlite3 *db)
{
    int rc;
    rc = sqlite3_create_function(db, "database_list", 0, SQLITE_UTF8, 0, database_list, 0, 0);
    return rc;
}

#if !SQLITE_CORE
int sqlite3_extension_init(sqlite3 *db, char **pzErrMsg,
                        const sqlite3_api_routines *pApi)
{
    SQLITE_EXTENSION_INIT2(pApi)
    return sqlite3PragmawrapperInit(db);
}
#endif

#endif
